@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2018 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -20,7 +20,11 @@
 #include <stdlib.h>
 #include <stdint.h>
 #include <unistd.h>
+#include <fstream>
 
+#ifdef HAVE_OPTRESET
+extern int optreset;
+#endif
 
 using namespace std;
 using namespace isc;
@@ -117,7 +121,10 @@ CommandOptions::reset() {
     mac_template_.assign(mac, mac + 6);
     duid_template_.clear();
     base_.clear();
+    mac_list_file_.clear();
+    mac_list_.clear();
     num_request_.clear();
+    exit_wait_time_ = 0;
     period_ = 0;
     drop_time_set_ = 0;
     drop_time_.assign(dt, dt + 2);
@@ -142,6 +149,7 @@ CommandOptions::reset() {
     diags_.clear();
     wrapped_.clear();
     server_name_.clear();
+    v6_relay_encapsulation_level_ = 0;
     generateDuidTemplate();
 }
 
@@ -205,11 +213,12 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
 
     std::ostringstream stream;
     stream << "perfdhcp";
+    int num_mac_list_files = 0;
 
     // In this section we collect argument values from command line
     // they will be tuned and validated elsewhere
-    while((opt = getopt(argc, argv, "hv46r:t:R:b:n:p:d:D:l:P:a:L:"
-                        "s:iBc1T:X:O:E:S:I:x:w:e:f:F:")) != -1) {
+    while((opt = getopt(argc, argv, "hv46A:r:t:R:b:n:p:d:D:l:P:a:L:M:"
+                        "s:iBc1T:X:O:E:S:I:x:W:w:e:f:F:")) != -1) {
         stream << " -" << static_cast<char>(opt);
         if (optarg) {
             stream << " " << optarg;
@@ -219,6 +228,19 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
             use_first_ = true;
             break;
 
+        // Simulate DHCPv6 relayed traffic.
+        case 'A':
+            // @todo: At the moment we only support simulating a single relay
+            // agent. In the future we should extend it to up to 32.
+            // See comment in https://github.com/isc-projects/kea/pull/22#issuecomment-243405600
+            v6_relay_encapsulation_level_ =
+                static_cast<uint8_t>(positiveInteger("-A<encapsulation-level> must"
+                                                     " be a positive integer"));
+            if (v6_relay_encapsulation_level_ != 1) {
+                isc_throw(isc::InvalidParameter, "-A only supports 1 at the moment.");
+            }
+            break;
+
         case '4':
             check(ipversion_ == 6, "IP version already set to 6");
             ipversion_ = 4;
@@ -286,7 +308,7 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
                 max_pdrop_.push_back(drop_percent);
             } else {
                 num_drops = positiveInteger("value of max drops number:"
-                                            " -d<value> must be a positive integer");
+                                            " -D<value> must be a positive integer");
                 max_drop_.push_back(num_drops);
             }
             break;
@@ -340,6 +362,19 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
                   boost::lexical_cast<std::string>(std::numeric_limits<uint16_t>::max()));
             break;
 
+        case 'M':
+            check(num_mac_list_files >= 1, "only -M option can be specified");
+            num_mac_list_files++;
+            mac_list_file_ = std::string(optarg);
+            loadMacs();
+            break;
+
+        case 'W':
+            exit_wait_time_ = nonNegativeInteger("value of exist wait time: "
+                                                  "-W<value> must not be a "
+                                                  "negative integer");
+            break;
+
         case 'n':
             num_req = positiveInteger("value of num-request:"
                                       " -n<value> must be a positive integer");
@@ -554,7 +589,7 @@ CommandOptions::decodeBase(const std::string& base) {
 
     // Currently we only support mac and duid
     if ((b.substr(0, 4) == "mac=") || (b.substr(0, 6) == "ether=")) {
-        decodeMac(b);
+        decodeMacBase(b);
     } else if (b.substr(0, 5) == "duid=") {
         decodeDuid(b);
     } else {
@@ -565,7 +600,7 @@ CommandOptions::decodeBase(const std::string& base) {
 }
 
 void
-CommandOptions::decodeMac(const std::string& base) {
+CommandOptions::decodeMacBase(const std::string& base) {
     // Strip string from mac=
     size_t found = base.find('=');
     static const char* errmsg = "expected -b<base> format for"
@@ -685,12 +720,45 @@ CommandOptions::convertHexString(const std::string& text) const {
     return ui;
 }
 
+void CommandOptions::loadMacs() {
+  std::string line;
+  std::ifstream infile(mac_list_file_.c_str());
+  while (std::getline(infile, line)) {
+    check(decodeMacString(line), "invalid mac in input");
+  }
+}
+
+bool CommandOptions::decodeMacString(const std::string& line) {
+  // decode mac string into a vector of uint8_t returns true in case of error.
+  std::istringstream s(line);
+  std::string token;
+  std::vector<uint8_t> mac;
+  while(std::getline(s, token, ':')) {
+    // Convert token to byte value using std::istringstream
+    if (token.length() > 0) {
+      unsigned int ui = 0;
+      try {
+        // Do actual conversion
+        ui = convertHexString(token);
+      } catch (isc::InvalidParameter&) {
+        return (true);
+      }
+      // If conversion succeeded store byte value
+      mac.push_back(ui);
+    }
+  }
+  mac_list_.push_back(mac);
+  return (false);
+}
+
 void
 CommandOptions::validate() const {
     check((getIpVersion() != 4) && (isBroadcast() != 0),
           "-B is not compatible with IPv6 (-6)");
     check((getIpVersion() != 6) && (isRapidCommit() != 0),
           "-6 (IPv6) must be set to use -c");
+    check(getIpVersion() == 4 && isUseRelayedV6(),
+          "Can't use -4 with -A, it's a V6 only option.");
     check((getIpVersion() != 6) && (getReleaseRate() != 0),
           "-F<release-rate> may be used with -6 (IPv6) only");
     check((getExchangeMode() == DO_SA) && (getNumRequests().size() > 1),
@@ -726,15 +794,6 @@ CommandOptions::validate() const {
           "-F<release-rate> is not compatible with -i");
     check((getExchangeMode() != DO_SA) && (isRapidCommit() != 0),
           "-i must be set to use -c");
-    check((getRate() == 0) && (getReportDelay() != 0),
-          "-r<rate> must be set to use -t<report>");
-    check((getRate() == 0) && (getNumRequests().size() > 0),
-          "-r<rate> must be set to use -n<num-request>");
-    check((getRate() == 0) && (getPeriod() != 0),
-          "-r<rate> must be set to use -p<test-period>");
-    check((getRate() == 0) &&
-          ((getMaxDrop().size() > 0) || getMaxDropPercentage().size() > 0),
-          "-r<rate> must be set to use -D<max-drop>");
     check((getRate() != 0) && (getRenewRate() + getReleaseRate() > getRate()),
           "The sum of Renew rate (-f<renew-rate>) and Release rate"
           " (-F<release-rate>) must not be greater than the exchange"
@@ -757,7 +816,8 @@ CommandOptions::validate() const {
     check((getTemplateFiles().size() < 2) && (getRequestedIpOffset() >= 0),
           "second/request -T<template-file> must be set to "
           "use -I<ip-offset>");
-
+    check((!getMacListFile().empty() && base_.size() > 0),
+          "Can't use -b with -M option");
 }
 
 void
@@ -871,6 +931,9 @@ CommandOptions::printCommandLine() const {
     if (use_first_) {
         std::cout << "use-first" << std::endl;
     }
+    if (!mac_list_file_.empty()) {
+        std::cout << "mac-list-file=" << mac_list_file_ << std::endl;
+    }
     for (size_t i = 0; i < template_file_.size(); ++i) {
         std::cout << "template-file[" << i << "]=" << template_file_[i] << std::endl;
     }
@@ -910,15 +973,17 @@ CommandOptions::printCommandLine() const {
 void
 CommandOptions::usage() const {
     std::cout <<
-        "perfdhcp [-hv] [-4|-6] [-e<lease-type>] [-r<rate>] [-f<renew-rate>]\n"
+        "perfdhcp [-hv] [-4|-6] [-A<encapsulation-level>] [-e<lease-type>]"
+        "         [-r<rate>] [-f<renew-rate>]\n"
         "         [-F<release-rate>] [-t<report>] [-R<range>] [-b<base>]\n"
         "         [-n<num-request>] [-p<test-period>] [-d<drop-time>]\n"
         "         [-D<max-drop>] [-l<local-addr|interface>] [-P<preload>]\n"
         "         [-a<aggressivity>] [-L<local-port>] [-s<seed>] [-i] [-B]\n"
-        "         [-c] [-1] [-T<template-file>] [-X<xid-offset>]\n"
-        "         [-O<random-offset] [-E<time-offset>] [-S<srvid-offset>]\n"
-        "         [-I<ip-offset>] [-x<diagnostic-selector>] [-w<wrapped>]\n"
-        "         [server]\n"
+        "         [-W<late-exit-delay>]\n"
+        "         [-c] [-1] [-M<mac-list-file>] [-T<template-file>]\n"
+        "         [-X<xid-offset>] [-O<random-offset] [-E<time-offset>]\n"
+        "         [-S<srvid-offset>] [-I<ip-offset>] [-x<diagnostic-selector>]\n"
+        "         [-w<wrapped>] [server]\n"
         "\n"
         "The [server] argument is the name/address of the DHCP server to\n"
         "contact.  For DHCPv4 operation, exchanges are initiated by\n"
@@ -979,6 +1044,11 @@ CommandOptions::usage() const {
         "    via which exchanges are initiated.\n"
         "-L<local-port>: Specify the local port to use\n"
         "    (the value 0 means to use the default).\n"
+        "-M<mac-list-file>: A text file containing a list of MAC addresses,\n"
+        "   one per line. If provided, a MAC address will be chosen randomly\n"
+        "   from this list for every new exchange. In the DHCPv6 case, MAC\n"
+        "   addresses are used to generate DUID-LLs. This parameter must not be\n"
+        "   used in conjunction with the -b parameter.\n"
         "-O<random-offset>: Offset of the last octet to randomize in the template.\n"
         "-P<preload>: Initiate first <preload> exchanges back to back at startup.\n"
         "-r<rate>: Initiate <rate> DORA/SARR (or if -i is given, DO/SA)\n"
@@ -994,6 +1064,9 @@ CommandOptions::usage() const {
         "-T<template-file>: The name of a file containing the template to use\n"
         "    as a stream of hexadecimal digits.\n"
         "-v: Report the version number of this program.\n"
+        "-W<time>: Specifies exit-wait-time parameter, that makes perfdhcp wait\n"
+        "    for <time> us after an exit condition has been met to receive all\n"
+        "    packets without sending any new packets. Expressed in microseconds.\n"
         "-w<wrapped>: Command to call with start/stop at the beginning/end of\n"
         "    the program.\n"
         "-x<diagnostic-selector>: Include extended diagnostics in the output.\n"
@@ -1018,15 +1091,21 @@ CommandOptions::usage() const {
         "    the exchange rate (given by -r<rate>).  Furthermore the sum of\n"
         "    this value and the renew-rate (given by -f<rate) must be equal\n"
         "    to or less than the exchange rate.\n"
+        "-A<encapsulation-level>: Specifies that relayed traffic must be\n"
+        "    generated. The argument specifies the level of encapsulation, i.e.\n"
+        "    how many relay agents are simulated. Currently the only supported\n"
+        "    <encapsulation-level> value is 1, which means that the generated\n"
+        "    traffic is an equivalent of the traffic passing through a single\n"
+        "    relay agent.\n"
         "\n"
-        "The remaining options are used only in conjunction with -r:\n"
+        "The remaining options are typically used in conjunction with -r:\n"
         "\n"
-        "-D<max-drop>: Abort the test if more than <max-drop> requests have\n"
-        "    been dropped.  Use -D0 to abort if even a single request has been\n"
-        "    dropped.  If <max-drop> includes the suffix '%', it specifies a\n"
-        "    maximum percentage of requests that may be dropped before abort.\n"
-        "    In this case, testing of the threshold begins after 10 requests\n"
-        "    have been expected to be received.\n"
+        "-D<max-drop>: Abort the test immediately if max-drop requests have\n"
+        "    been dropped.  max-drop must be a positive integer. If max-drop\n"
+        "    includes the suffix '%', it specifies a maximum percentage of\n"
+        "    requests that may be dropped before abort. In this case, testing\n"
+        "    of the threshold begins after 10 requests have been expected to\n"
+        "    be received.\n"
         "-n<num-request>: Initiate <num-request> transactions.  No report is\n"
         "    generated until all transactions have been initiated/waited-for,\n"
         "    after which a report is generated and the program terminates.\n"
@@ -1056,6 +1135,5 @@ CommandOptions::version() const {
     std::cout << "VERSION: " << VERSION << std::endl;
 }
 
-
-} // namespace perfdhcp
-} // namespace isc
+}  // namespace perfdhcp
+}  // namespace isc
