@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2018 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -214,6 +214,8 @@ SrvConfig::toElement() const {
     ElementPtr result = Element::createMap();
     // DhcpX global map
     ElementPtr dhcp = Element::createMap();
+    // Set user-context
+    contextToElement(dhcp);
     // Set decline-probation-period
     dhcp->set("decline-probation-period",
               Element::create(static_cast<long long>(decline_timer_)));
@@ -234,27 +236,91 @@ SrvConfig::toElement() const {
     dhcp->set("option-data", cfg_option_->toElement());
 
     // Set subnets and shared networks.
-    ConstElementPtr subnets;
-    if (family == AF_INET) {
-        subnets = cfg_subnets4_->toElement();
-        dhcp->set("subnet4", subnets);
 
-        ConstElementPtr shared_networks = cfg_shared_networks4_->toElement();
+    // We have two problems to solve:
+    //   - a subnet is unparsed once:
+    //       * if it is a plain subnet in the global subnet list
+    //       * if it is a member of a shared network in the shared network
+    //         subnet list
+    //   - unparsed subnets must be kept to add host reservations in them.
+    //     Of course this can be done only when subnets are unparsed.
+
+    // The list of all unparsed subnets
+    std::vector<ElementPtr> sn_list;
+
+    if (family == AF_INET) {
+        // Get plain subnets
+        ElementPtr plain_subnets = Element::createList();
+        const Subnet4Collection* subnets = cfg_subnets4_->getAll();
+        for (Subnet4Collection::const_iterator subnet = subnets->cbegin();
+             subnet != subnets->cend(); ++subnet) {
+            // Skip subnets which are in a shared-network
+            SharedNetwork4Ptr network;
+            (*subnet)->getSharedNetwork(network);
+            if (network) {
+                continue;
+            }
+            ElementPtr subnet_cfg = (*subnet)->toElement();
+            sn_list.push_back(subnet_cfg);
+            plain_subnets->add(subnet_cfg);
+        }
+        dhcp->set("subnet4", plain_subnets);
+
+        // Get shared networks
+        ElementPtr shared_networks = cfg_shared_networks4_->toElement();
         dhcp->set("shared-networks", shared_networks);
 
+        // Get subnets in shared network subnet lists
+        const std::vector<ElementPtr> networks = shared_networks->listValue();
+        for (auto network = networks.cbegin();
+             network != networks.cend(); ++network) {
+            const std::vector<ElementPtr> sh_list =
+                (*network)->get("subnet4")->listValue();
+            for (auto subnet = sh_list.cbegin();
+                 subnet != sh_list.cend(); ++subnet) {
+                sn_list.push_back(*subnet);
+            }
+        }
+
     } else {
-        subnets = cfg_subnets6_->toElement();
-        dhcp->set("subnet6", subnets);
+        // Get plain subnets
+        ElementPtr plain_subnets = Element::createList();
+        const Subnet6Collection* subnets = cfg_subnets6_->getAll();
+        for (Subnet6Collection::const_iterator subnet = subnets->cbegin();
+             subnet != subnets->cend(); ++subnet) {
+            // Skip subnets which are in a shared-network
+            SharedNetwork6Ptr network;
+            (*subnet)->getSharedNetwork(network);
+            if (network) {
+                continue;
+            }
+            ElementPtr subnet_cfg = (*subnet)->toElement();
+            sn_list.push_back(subnet_cfg);
+            plain_subnets->add(subnet_cfg);
+        }
+        dhcp->set("subnet6", plain_subnets);
 
-        ConstElementPtr shared_networks = cfg_shared_networks6_->toElement();
+        // Get shared networks
+        ElementPtr shared_networks = cfg_shared_networks6_->toElement();
         dhcp->set("shared-networks", shared_networks);
+
+        // Get subnets in shared network subnet lists
+        const std::vector<ElementPtr> networks = shared_networks->listValue();
+        for (auto network = networks.cbegin();
+             network != networks.cend(); ++network) {
+            const std::vector<ElementPtr> sh_list =
+                (*network)->get("subnet6")->listValue();
+            for (auto subnet = sh_list.cbegin();
+                 subnet != sh_list.cend(); ++subnet) {
+                sn_list.push_back(*subnet);
+            }
+        }
     }
     // Insert reservations
     CfgHostsList resv_list;
     resv_list.internalize(cfg_hosts_->toElement());
-    const std::vector<ElementPtr>& sn_list = subnets->listValue();
-    for (std::vector<ElementPtr>::const_iterator subnet = sn_list.begin();
-         subnet != sn_list.end(); ++subnet) {
+    for (std::vector<ElementPtr>::const_iterator subnet = sn_list.cbegin();
+         subnet != sn_list.cend(); ++subnet) {
         ConstElementPtr id = (*subnet)->get("id");
         if (isNull(id)) {
             isc_throw(ToElementError, "subnet has no id");
@@ -276,12 +342,11 @@ SrvConfig::toElement() const {
     // Set lease-database
     CfgLeaseDbAccess lease_db(*cfg_db_access_);
     dhcp->set("lease-database", lease_db.toElement());
-    // Set hosts-database
+    // Set hosts-databases
     CfgHostDbAccess host_db(*cfg_db_access_);
-    // @todo accept empty map
-    ConstElementPtr hosts_database = host_db.toElement();
-    if (hosts_database->size() > 0) {
-        dhcp->set("hosts-database", hosts_database);
+    ConstElementPtr hosts_databases = host_db.toElement();
+    if (hosts_databases->size() > 0) {
+        dhcp->set("hosts-databases", hosts_databases);
     }
     // Set host-reservation-identifiers
     ConstElementPtr host_ids;
@@ -297,7 +362,7 @@ SrvConfig::toElement() const {
     }
     // Set control-socket (skip if null as empty is not legal)
     if (!isNull(control_socket_)) {
-        dhcp->set("control-socket", control_socket_);
+        dhcp->set("control-socket", UserContext::toElement(control_socket_));
     }
     // Set client-classes
     ConstElementPtr client_classes = class_dictionary_->toElement();
